#ifndef __CONV_UTILS_H__
#define __CONV_UTILS_H__

#include < vcclr.h >
#include <string>
#include <vector>
#include <list>

using namespace System;
using namespace System::Collections::Generic;

namespace MySQL {

static std::string replace_string(const std::string &s,
                                  const std::string &from,
                                  const std::string &to)
{
  std::string::size_type p;
  std::string ss, res;

  ss= s;
  p= ss.find(from);
  while (p != std::string::npos)
  {
    res+= ss.substr(0, p) + to;
    ss= ss.substr(p+from.size());
    p= ss.find(from);
  }
  res+= ss;

  return res;
}

static String^ CppStringToNative(const std::string &str)
{
  // TODO optimize here - do not copy string creating std::string
  std::string res= replace_string(str, "\n", "\r\n");

  // TODO is there a faster way to cache this encoding obj?
  return gcnew String(res.c_str(), 0, (int) res.size(), gcnew System::Text::UTF8Encoding(true, true));
}

static std::string NativeToCppString(String^ str)
{
  if (!str)
    return NULL;

  // Pin memory so GC can't move it while native function is called
  pin_ptr<const wchar_t> wch = PtrToStringChars(str);

  // Conversion to char* :
  // Can just convert wchar_t* to char* using one of the 
  // conversion functions such as: 
  // WideCharToMultiByte()
  // wcstombs_s()
  // ... etc

  int sizeInBytes = ((str->Length + 1) * 4); // utf8 can have up to 4 bytes per char (not 2)
  errno_t err = 0;
  char    *ch = new char[sizeInBytes];

  size_t convertedChars = WideCharToMultiByte(CP_UTF8, 0, wch, -1, ch, sizeInBytes, 0, 0);
  //if (err != 0)
  //  throw ...

  // compress \r\n to \n
  char *from, *to;

  for (from= to= ch; *from != '\0'; ++from, ++to)
  {
    if (*from == '\r' && *(from+1)=='\n')
      ++from;
    *to= *from;
  }
  *to= 0;

  std::string result(ch);
  delete[] ch;
  return result;
}

static List<String^>^ CppStringListToNative(const std::vector<std::string> &input)
{
  int cap = static_cast<int>(input.size());
  List<String^>^ result= gcnew List<String^>(cap);
  std::vector<std::string>::const_iterator e= input.end();
  
  for (std::vector<std::string>::const_iterator i= input.begin(); i != e; i++)
    result->Add(CppStringToNative(*i));
  return result;
}

static std::vector<std::string> NativeToCppStringList(List<String^>^ input)
{
  std::vector<std::string> result;
  result.reserve(input->Count);
  for each(String ^str_item in input)
    result.push_back(NativeToCppString(str_item));
  return result;
}

static std::list<std::string> NativeToCppStringList2(List<String^>^ input)
{
  std::list<std::string> result;
  for each(String ^str_item in input)
    result.push_back(NativeToCppString(str_item));
  return result;
}

static List<String^>^ CppStringListToNative2(const std::list<std::string> &input)
{
  int cap = static_cast<int>(input.size());
  List<String^>^ result= gcnew List<String^>(cap);
  std::list<std::string>::const_iterator e= input.end();
  
  for (std::list<std::string>::const_iterator i= input.begin(); i != e; i++)
    result->Add(CppStringToNative(*i));
  return result;
}

static List<int>^ CppVectorToIntList(const std::vector<int>& input)
{
  typedef const std::vector<int> SourceContainerType;
  typedef List<int> TargetContainerType;

  TargetContainerType^ result= gcnew TargetContainerType(static_cast<int>(input.size()));
  SourceContainerType::const_iterator e= input.end();
  
  for(SourceContainerType::const_iterator i= input.begin(); i != e; i++)
    result->Add(*i);

  return result;
}

static std::vector<int> IntListToCppVector(List<int>^ input)
{
  typedef const List<int>^ SourceContainerType;
  typedef std::vector<int> TargetContainerType;

  TargetContainerType result;
  result.reserve(static_cast<int>(input->Count));
  
  for each(int listitem in input)
    result.push_back(listitem);

  return result;
}

} // namespace MySQL

#endif // __CONV_UTILS_H__
