﻿using System;
using System.Collections.Generic;
using System.Text;
using ScintillaNet;
using System.Windows;
using System.Windows.Forms;
using System.Drawing;


namespace MySQL.Grt.Db.Sql
{
  public class SqlEditor : Scintilla
  {
    private Sql_editor sqlEditorBE;
    private GrtManager grtManager;
    private Timer timer;

    public delegate void BackgroundActionDelegate();
    public BackgroundActionDelegate BackgroundAction;

    public SqlEditor(GrtManager grtManager)
    {
      GrtManager = grtManager;
      Initialize();
    }

    public GrtManager GrtManager
    {
      get { return grtManager; }
      set { grtManager = value; }
    }

    public Sql_editor BE
    {
      get { return sqlEditorBE; }
      set
      {
        sqlEditorBE = value;
        sqlEditorBE.sql_parser_err_cb(ProcessSyntaxError);

        SuspendLayout();

        // customized parameters
        ConfigurationManager.Language = sqlEditorBE.string_option("ConfigurationManager.Language");
        Indentation.IndentWidth = sqlEditorBE.int_option("Indentation.IndentWidth");
        Indentation.TabWidth = sqlEditorBE.int_option("Indentation.TabWidth");
        Indentation.UseTabs = sqlEditorBE.int_option("Indentation.UseTabs") != 0;
        //! todo: virtualize the rest params

        ResumeLayout();
      }
    }

    protected virtual void Initialize()
    {
      BeginInit();

      Dock = DockStyle.Fill;
      Printing.PageSettings.Color = false;
      UseFont = true;
      //!DocumentChange += new System.EventHandler<NativeScintillaEventArgs>(sqlEditorControl_Change);
      Font = grtManager.get_font_option("workbench.general.Editor:Font");
      Margins.Margin0.Width = 35; // line numbers
      Margins.Margin1.Width = 16; // markers
      Margins.Margin2.Width = 16; // indicators
      Indentation.BackspaceUnindents = true;
      Indentation.SmartIndentType = SmartIndent.Simple;

      // errors indication
      Indicator errInd = Indicators[0];
      errInd.Color = Color.Red;
      errInd.Style = ScintillaNet.IndicatorStyle.Squiggle;
      errInd.IsDrawnUnder = true;

      errInd = Indicators[1];
      errInd.Color = Color.Red;
      errInd.Style = ScintillaNet.IndicatorStyle.RoundBox;
      errInd.IsDrawnUnder = true;

      Marker marker = Markers[0];
      marker.BackColor = Color.Red;

      // fixes improper ViewportSize from start.
      // once row is stretched enough to make thumb appear this thumb becomes irreversible.
      Scrolling.HorizontalWidth = 1;

      NativeInterface.SetCodePage((int)Constants.SC_CP_UTF8);
      EndOfLine.Mode = EndOfLineMode.Crlf;

      timer = new Timer();
      timer.Interval = 2000;
      timer.Tick += new EventHandler(BackgroundActionTimer);

      BackgroundAction = CheckSyntax;

      TextDeleted += new System.EventHandler<ScintillaNet.TextModifiedEventArgs>(TextModified);
      TextInserted += new System.EventHandler<ScintillaNet.TextModifiedEventArgs>(TextModified);

      EndInit();
    }

    public String SqlText
    {
      get { return Text; }
      set
      {
        if (Text == value)
          return;

        BackgroundActionDelegate backgroundAction = BackgroundAction;
        BackgroundAction = null;

        int caretPos = Selection.End;
        Text = value;
        Selection.Start = Selection.End = caretPos;

        BackgroundAction = backgroundAction;
      }
    }

    private void TextModified(object sender, ScintillaNet.TextModifiedEventArgs e)
    {
      if (null != BackgroundAction)
      {
        timer.Stop();
        timer.Start();
      }
    }

    private void BackgroundActionTimer(Object obj, EventArgs args)
    {
      IsDirty = false;
      if (null != BackgroundAction)
        BackgroundAction();
    }

    public void CheckSyntax()
    {
      ResetSyntaxErrors();
      if (null == sqlEditorBE)
        return;
      sqlEditorBE.sql(Text);
      sqlEditorBE.check_syntax();
    }

    public int ProcessSyntaxError(int line, int errTokLinePos, int errTokLen, String msg)
    {
      line -= 1;
      try
      {
        errTokLinePos += Lines[line].Range.Start;
        Range range = new Range(errTokLinePos, errTokLinePos + errTokLen, this);
        range.SetIndicator(0);
        range.SetIndicator(1);
        Lines[line].AddMarker(0);
      }
      catch (Exception) { }
      return 0;
    }

    public void ResetSyntaxErrors()
    {
      timer.Stop();
      Range range = new Range(0, RawText.Length, this);
      range.ClearIndicator(0);
      range.ClearIndicator(1);
      Markers.DeleteAll(Markers[0]);
    }
  }
}
