/* 
 * © 2009 Sun Microsystems, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
/**
 * Windows front end implementation of the mforms Utilities APIs.
 */

#include "stdafx.h"
#include "wf_Utilities.h"

using namespace Windows::Forms;
using namespace System::Text;

using namespace MySQL::Forms;

//--------------------------------------------------------------------------------------------------

UtilitiesImpl::UtilitiesImpl()
{
}

//--------------------------------------------------------------------------------------------------

int UtilitiesImpl::show_message(const string &title, const string &text, const string &ok, 
  const string &cancel, const string &other)
{
  return 0;
}

//--------------------------------------------------------------------------------------------------

int UtilitiesImpl::show_error(const string &title, const string &text, const string &ok, 
  const string &cancel, const string &other)
{
  return 0;
}

//--------------------------------------------------------------------------------------------------

int UtilitiesImpl::show_warning(const string &title, const string &text, const string &ok, 
  const string &cancel, const string &other)
{
  return 0;
}

//--------------------------------------------------------------------------------------------------

/**
 * Places the given string on the clipboard.
 * 
 * @param content The text to be placed on the clipboard. It is assume its encoding is UTF-8.
 * @note The given text is transformed to be Windows compliant. That means it is converted to UTF-16 and
 *       LF only line breaks are converted to Windows-typical CRLF combinations.
 */
void UtilitiesImpl::set_clipboard_text(const string &content)
{
  StringBuilder^ builder= gcnew StringBuilder(CppStringToNative(content));
  builder->Replace("\n", Environment::NewLine);

  //String^ unicode= Encoding::Unicode->GetString(Encoding::UTF8->GetBytes(builder->ToString()));
  //Clipboard::SetData(DataFormats::UnicodeText, unicode);

  Clipboard::SetText(builder->ToString());
}

//--------------------------------------------------------------------------------------------------

/**
 * Returns the current text on the clipboard, if there is any.
 * 
 * @result If there is text on the clipboard (ANSI or Unicode) it is returned as UTF-8 string.
 * @note The returned text gets all CRLF Windows line breaks converted to pure LF.
 */
std::string UtilitiesImpl::get_clipboard_text()
{
  String^ unicode= (String^) Clipboard::GetData(DataFormats::UnicodeText);
  if (unicode == nullptr)
    return "";

  String^ content= Encoding::UTF8->GetString(Encoding::Unicode->GetBytes(unicode));
  StringBuilder^ builder= gcnew StringBuilder(content);
  builder->Replace(Environment::NewLine, "\n");

  return NativeToCppString(builder->ToString());
}

//--------------------------------------------------------------------------------------------------

void UtilitiesImpl::open_url(const string &url)
{
  try
  {
    System::Diagnostics::Process::Start(CppStringToNative(url));
  }
  catch (Exception ^e)
  {
    MessageBox::Show(e->Message->ToString(), "Error Opening Browser", 
      MessageBoxButtons::OK, MessageBoxIcon::Error, MessageBoxDefaultButton::Button1);
  }
}

